package euc_utf8;

########################################################################
# EUCKR <-> UTF8 Code Conversion V0.3 1999.05.12
#
# EUCKR : It has KSC5601 + ASCII but I just meant as UnifiedWansungHangul by Microsoft (cp949)
# UTF8 : 1 or 2 bytes form of UCS2 in safe code area
#
#	EUCKR: xxxx xxxx    xxxx xxxx (2 bytes)
#	UCS2 : xxxx xxxx    xxxx xxxx (2 bytes)
#	UTF8 : 1110 xxxx    10xx xxxx    10xx xxxx (3 bytes)
#
# This requires 2 code table library files
#		euc_ucs2.map
#		ucs2_euc.map
#
# There are 2 subs,
#		UTF8_EUCKR() : argument is a REFerence to the string, and no return value
#		EUCKR_UTF8() : argument is a REFerence to the string, and no return value
#		(so if you want to save the original string, save its copy before call the sub)		
#
#		* All the non allocated codes are converted to 0xA1E0 for EUCKR -> UCS2(0x25A1)
#
# Copyright by Gillysoft 1999 hklife@soback.kornet21.net
########################################################################

########################################################################
# EXAMPLE
#
# $string_euckr = "ݶ or c氢";
# $string_utf8 = "?시콜라 or ?방각하";
#
# require 'euc_utf8.pm';
#
# euc_utf8::EUCKR_UTF8(\$string_euckr);
# print $string_euckr,"\n";
#
# euc_utf8::UTF8_EUCKR(\$string_euckr);
# print $string_euckr;
#
########################################################################


########################################################################
sub UTF8_EUCKR {
	require 'ucs2_euc.map';

	my $convert = sub {
		my ($ch1, $ch2, $ch3) = shift =~ /(.)(.)(.)/; #  UTF8 -> UCS2
		my $uni_1 = uc unpack ('H*', chr(((hex(unpack('H*', $ch2)) >> 2) & 15) | (hex(unpack ('h', $ch1)) << 4)));
		my $uni_2 = uc unpack ('H*', chr( (hex(unpack ('H*', $ch2)) << 6) | (hex(unpack ('H*', $ch3)) & 63) ));
		return $TABLE{$uni_1.$uni_2}; #  UCS2 -> UnifiedWansungHangul
	};
	
	${ +shift } =~ s/([\xE0-\xEF][\x80-\xBF]{2})/&$convert($1)/ge;
}


########################################################################
sub EUCKR_UTF8 {
	require 'euc_ucs2.map';

	my $convert = sub {
		## UnifiedWansungHangul -> UCS2 code in hex
		my ($ch1, $ch2) = map { pack('C', hex $_) } $TABLE{+shift} =~ /(..)(..)/;
		$ch1 = chr(0x25), $ch2 = chr(0xA1) if $ch1 eq '' && $ch2 eq ''; # 0xA1E0 -> 0x25A1

		## UCS2 code -> UTF8
		my $utf8_1 = chr( (hex(unpack ('H', $ch1))) | 224 );
		my $utf8_2 = chr( (hex(unpack ('h', $ch1)) << 2) | 128 | (hex(unpack ('H*', $ch2)) >> 6) );
		my $utf8_3 = chr( (hex(unpack ('H2', $ch2)) &  63) | 128 );
		return $utf8_1.$utf8_2.$utf8_3;
	};

	${ +shift } =~ s/([\x81-\xFE][\x41-\xFE])/&$convert($1)/ge;
}

1;
__END__

